package db;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.ArrayList;

import model.Observation;
import cede.Matrix;

/**
 * Handles all insert method into the database
 *
 */

public class DataInserter {
	
	private static DataInserter instance = null;
	public Database db = Database.getInstance();
	public static boolean shouldLog = true;
	
	public static DataInserter getInstance() { 
		return instance==null? (instance=new DataInserter()):instance;
	}

	private DataInserter() {
		
	}
	
	public void cleanMissingValueTable() {
		log("Missing values table is cleaned. ");
		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
			} 
			String tsQuery = "DELETE FROM missing_values";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.executeQuery();
			tsStmt.close();

		} catch (SQLException e) {
			e.printStackTrace();
		}
	}

	public void insertRecoveredValue(int ts_id, ArrayList<Observation> recovValues) { 
		log("Inserting recovered values into time series: " + ts_id);
		try {
			Connection conn = db.newConnection();
			if (conn==null) {
				System.out.println("Connection null");
			}
			conn.setAutoCommit(false);

			String tsQuery = "UPDATE missing_values SET val_CD=? WHERE series_id=? AND ts=?";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);

			for(Observation mo: recovValues) {
				tsStmt.setFloat(1, mo.getValue());
				tsStmt.setInt(2, ts_id);
				tsStmt.setInt(3, mo.getTimeStamp());						
				tsStmt.executeQuery();
			}
			conn.commit();
		} catch (SQLException e) {


			e.printStackTrace();
		}
	}

	public void insertRecoveredValue(int ts_id, Matrix recovValues) { 
		try {
			Connection conn = db.newConnection();
			if (conn==null) {
				System.out.println("Connection null");
			}
			conn.setAutoCommit(false);

			String tsQuery = "UPDATE missing_values SET val_CD=? WHERE series_id=? AND ts=?";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);

			for(int i = 0; i < recovValues.getRows(); i++) {
				tsStmt.setFloat(1, recovValues.get(i, 1));
				tsStmt.setInt(2, ts_id);
				tsStmt.setInt(3, (int) recovValues.get(i, 0));						
				tsStmt.executeQuery();		
				//				tsStmt.close();	

			}
			conn.commit();
		} catch (SQLException e) {


			e.printStackTrace();
		}
	}

	public void insertORupdateMissingValues(int ts_id, ArrayList<Observation> missingValues) {
		log("Inserting missing values of time series " + ts_id);
		try {
			Connection conn = db.newConnection();
			if (conn==null) {
				System.out.println("Connection null");
			}
			conn.setAutoCommit(false);

			String tsQuery = "INSERT INTO missing_values VALUES (?, ?, 0, ?, 0)";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);

			for(Observation mo: missingValues) {
				tsStmt.setInt(1, ts_id);
				tsStmt.setInt(2, mo.getTimeStamp());
				tsStmt.setFloat(3, mo.getValue());
				tsStmt.executeQuery();	
			}
			conn.commit();
		} catch (SQLException e) {
			e.printStackTrace();
		}	
	}

	public void insertORupdateMissingValues(int ts_id, Matrix missingValues) {
		try {
			Connection conn = db.newConnection();
			if (conn==null) {
				System.out.println("Connection null");
			}
			conn.setAutoCommit(false);

			String tsQuery = "INSERT INTO missing_values VALUES (?, ?, 0, ?, 0)";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);

			for(int i = 0; i < missingValues.getRows(); i++) {

				tsStmt.setInt(1, ts_id);
				tsStmt.setInt(2, (int) missingValues.get(i, 0));					
				tsStmt.setFloat(3, missingValues.get(i, 1));
				tsStmt.executeQuery();		
				//				tsStmt.close();	

			}
			conn.commit();
		} catch (SQLException e) {


			e.printStackTrace();
		}
	}
	
	private static void log(String text) {
		if (shouldLog) {
			System.out.println(text);
		}
	}
	
}
