package db;

import gui.Controller;
import gui.DisplayController;

import java.awt.Color;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import javax.swing.text.html.HTMLDocument.Iterator;

import org.jfree.data.xy.XYSeries;

import model.Observation;
import model.TimeSeries;
import model.TimeSeriesSet;
import cede.Matrix;

/**
 * Contains all the methods, which selects data from the database
 *
 */

public class DataLoader {

	private static DataLoader instance = null;
	public static boolean shouldLog = true;
	public Database db = Database.getInstance();
	private Controller c;
	private DisplayController dc;
	
	public static DataLoader getInstance() { 
		return instance==null? (instance=new DataLoader()):instance;
	}

	private DataLoader() {
		
	}
		
	public int queryingForGranularity(int ts_id){
		int granul = 0;

		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return 0;
			}
			String tsQuery = "SELECT granul FROM series WHERE id = ?";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, ts_id);
			ResultSet tsRs = tsStmt.executeQuery();
//			tsStmt.close();

			if(tsRs.next()) {
				granul = tsRs.getInt(1);
			} 
		} catch (SQLException e) {
			e.printStackTrace();
			return 0;
		}

		return granul;
	}


	public ArrayList<Observation> loadPreProcTSwithMissingVal(int ts_id, int windowStart, int windowEnd) {
		System.out.println("Loading data of " + ts_id + " ...");
		ArrayList<Observation> list = new ArrayList<Observation>();

		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
			}

			String tsQuery = "(SELECT  ts, val as value FROM observation WHERE series_id = ? and ts between ? and ?) UNION (select ts, val_mis as value FROM missing_values WHERE series_id = ?)";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, ts_id);
			tsStmt.setInt(2, windowStart);
			tsStmt.setInt(3, windowEnd);
			tsStmt.setInt(4, ts_id);
			ResultSet tsRs = tsStmt.executeQuery();

			while (tsRs.next()) { 
				Observation mo = new Observation();
				mo.setIndex(0);
				mo.setTimeStamp(tsRs.getInt(1));
				mo.setValue(tsRs.getFloat(2));
				list.add(mo);
			}

		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}

		return list;	
	}

	public ArrayList<Observation> loadRecoveredTimeSeries(int ts_id, int windowStart, int windowEnd) {
		ArrayList<Observation> list = new ArrayList<Observation>();

		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
			}

			String tsQuery = "(SELECT  ts, val as value FROM observation WHERE series_id = ? and ts between ? and ?) UNION (select ts, val_cd as value FROM missing_values WHERE series_id = ?)";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, ts_id);
			tsStmt.setInt(2, windowStart);
			tsStmt.setInt(3, windowEnd);
			tsStmt.setInt(4, ts_id);
			ResultSet tsRs = tsStmt.executeQuery();

			while (tsRs.next()) { 
				Observation mo = new Observation();
				mo.setIndex(0);
				mo.setTimeStamp(tsRs.getInt(1));
				mo.setValue(tsRs.getFloat(2));
				list.add(mo);
			}

		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}

		return list;	
	}
	
	public Matrix loadPreprocessedTimeSeries(int ts_id, int windowStart, int windowEnd) {

		log("Loading preprocessed timeseries: " + ts_id + " starts ...");

		int gran = queryingForGranularity(ts_id);
		
		if(gran == 2) {
			if(windowStart % 2 == 1) {
				windowStart = windowStart+1;
			}
			if(windowEnd % 2 == 1) {
				windowEnd = windowEnd-1;
			}
		}
		
		int rowNum = (windowEnd-windowStart)/gran+1;

		System.out.println("A rownum: " + rowNum);

		int i = 0;
		Matrix m = new Matrix(rowNum,1);
		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
			}

			String tsQuery = "(SELECT  ts, val as value FROM observation WHERE series_id = ? and ts between ? and ?) UNION " +
					"(select ts, val_lin_interpol as value FROM missing_values WHERE series_id = ?)";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, ts_id);
			tsStmt.setInt(2, windowStart);
			tsStmt.setInt(3, windowEnd);
			tsStmt.setInt(4, ts_id);
			ResultSet tsRs = tsStmt.executeQuery();


			while (i < rowNum) {			
				tsRs.next();
				if(ts_id == 139) {
				System.out.println("Az i rtke: " + i);
				System.out.println("tsRs.getFloat(2)  " + tsRs.getFloat(2));
				}
				m.set(i, 0, tsRs.getFloat(2));
				i++;
			}	

		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}

		log("Loading preprocessed data timeseries: " + ts_id + " is done.");
		return m;
	}
	
	public int rowNumberBetweenTimestamps(int ts_id, int windowStart, int windowEnd) {
		int rowNum = 0;
		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return 0;
			}	

			String tsQuery = "SELECT COUNT (*) FROM observation where series_id = ? and ts between ? and ?";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, ts_id);
			tsStmt.setInt(2, windowStart);
			tsStmt.setInt(3, windowEnd);
			ResultSet tsRs = tsStmt.executeQuery();
			if(tsRs.next()) {
				rowNum = tsRs.getInt(1);
			} 

		} catch (SQLException e) {
			e.printStackTrace();
			return 0;
		}

		return rowNum;		
	}

	public Matrix loadTimeSeriesForPreprocessing(int ts_id, int windowStart, int windowEnd) {
		log("Loading data from timeseries " + ts_id + " for preprocessing.");

		int rowNum = rowNumberBetweenTimestamps(ts_id, windowStart, windowEnd);

		int i = 0;
		Matrix m = new Matrix(rowNum,2);
		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return null;
			}			

			log("Querying database for ID "+ ts_id);
			String tsQuery = "SELECT ts, val FROM observation WHERE series_id = ? and ts between ? and ? ORDER BY ts";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, ts_id);
			tsStmt.setInt(2, windowStart);
			tsStmt.setInt(3, windowEnd);
			ResultSet tsRs = tsStmt.executeQuery();

			do {			
				tsRs.next();
				m.set(i, 0, tsRs.getFloat(1));
				m.set(i, 1, tsRs.getFloat(2));
				i++;
			} while (i < rowNum);


		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}

		return m;
	}

	public XYSeries loadTimeSeriesForDisplay(int ts_id, int windowStart, int windowEnd) {
		logdc("Loading data from timeseries " + ts_id + " ...");
		int granul = -1;
		PreparedStatement tsStmt;
		ArrayList<Observation> list = new ArrayList<Observation>();
		XYSeries timeseries = new XYSeries("TimeSeries_" + Integer.toString(ts_id));

		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return null;
			}		
			
			String tsQuery1 = "SELECT granul FROM series WHERE id = ?";
			PreparedStatement tsStmt1= conn.prepareStatement(tsQuery1);
			tsStmt1.setInt(1, ts_id);
			ResultSet tsRs1 = tsStmt1.executeQuery();


			if(tsRs1.next()) {
				granul = tsRs1.getInt(1);
			} 

			log("Querying database for ID "+ ts_id);
			
			if(windowStart == -1 && windowEnd == -1) {
				String tsQuery = "SELECT ts, val FROM observation WHERE series_id = ? ORDER BY ts";
				tsStmt= conn.prepareStatement(tsQuery);
				tsStmt.setInt(1, ts_id);
			} else {
				String tsQuery = "SELECT ts, val FROM observation WHERE series_id = ? and ts between ? and ? ORDER BY ts";
				tsStmt= conn.prepareStatement(tsQuery);
				tsStmt.setInt(1, ts_id);
				tsStmt.setInt(2, windowStart);
				tsStmt.setInt(3, windowEnd);
			}
			
			
			ResultSet tsRs = tsStmt.executeQuery();
			

			while (tsRs.next()) { 
				Observation mo = new Observation();
				mo.setTimeStamp(tsRs.getInt(1));
				mo.setValue(tsRs.getFloat(2));
				list.add(mo);
			}
		
			
			for(int i=0; i < list.size()-1; i++) {
				int ts = list.get(i).getTimeStamp();
				int nexts = list.get(i+1).getTimeStamp();
				timeseries.add(ts, list.get(i).getValue());
				if(nexts-ts > 2) {
					timeseries.add((ts+granul), null);
//					System.out.println("ts+granul = " + (ts+granul));
				} else {
					
				}
			}
			

		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}

		logdc("Loading from " + ts_id + " is done.");
		return timeseries;
	}
	
	
	public XYSeries loadRecoveredTimeSeriesforDisplay(int ts_id, int windowStart, int windowEnd) {
		log("Loading recovered data from timeseries " + ts_id);

		XYSeries timeseries = new XYSeries("TimeSeries_" + Integer.toString(ts_id));
		
		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return null;
			}	
			
			String tsQuery = "(SELECT  ts, val as value FROM observation WHERE series_id = ? and ts between ? and ?) UNION (select ts, val_cd as value FROM missing_values WHERE series_id = ?)";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, ts_id);
			tsStmt.setInt(2, windowStart);
			tsStmt.setInt(3, windowEnd);
			tsStmt.setInt(4, ts_id);
			ResultSet tsRs = tsStmt.executeQuery();

			while (tsRs.next()) { 
				timeseries.add(tsRs.getInt(1), tsRs.getFloat(2));
			}
		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}

		log("Loading from recovered timeseries " + ts_id + " is done.");
		
		return timeseries;
		
	}

	
	
	public ArrayList<Observation> loadTimeSeries(int ts_id) {
		log("Loading data from timeseries " + ts_id);

		ArrayList<Observation> list = new ArrayList<Observation>();

		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return null;
			}			

			log("Querying database for ID "+ ts_id);
			String tsQuery = "SELECT ts, val FROM observation WHERE series_id = ? ORDER BY ts";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, ts_id);
			ResultSet tsRs = tsStmt.executeQuery();

			log("From the DB side it is done.");
			
			while (tsRs.next()) { 
				Observation mo = new Observation();
				mo.setIndex(0);
				mo.setTimeStamp(tsRs.getInt(1));
				mo.setValue(tsRs.getFloat(2));
				list.add(mo);
			}

		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}

		log("Loading from " + ts_id + " is done.");
		return list;	
	}

	public int getLastTS(int ts_id) {
		int lastTS = 0;

		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return 0;
			}
			String tsQuery = "SELECT MAX (ts) FROM observation where series_id = ?";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, ts_id);
			ResultSet tsRs = tsStmt.executeQuery();

			if(tsRs.next()) {
				lastTS = tsRs.getInt(1);
			} 

		} catch (SQLException e) {
			e.printStackTrace();
			return 0;
		}

		return lastTS;
	}

	public float getMaxValueFromMoreTs(ArrayList<Integer> timeseriesList, int windowStart, int windowEnd) {
		float maxVal = 0;
		PreparedStatement tsStmt;
		StringBuffer sbf = new StringBuffer();
		for(Integer i: timeseriesList) {
			sbf.append(Integer.toString(i) + ",");
		}
		
		String tslistStr = sbf.toString();
		tslistStr = tslistStr.substring(0, tslistStr.length() - 1);

		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return 0;
			}
			
			if(windowStart == -1 && windowEnd == -1) {
				String tsQuery = "SELECT MAX(maximum) FROM (SELECT MAX(val) as maximum FROM observation o WHERE series_id in (" + tslistStr + "))";
				System.out.println("A maxVal tsQuery: " + tsQuery);		
				 tsStmt= conn.prepareStatement(tsQuery);
			} else {
			
			String tsQuery = "SELECT MAX(maximum) FROM ((SELECT MAX(val) as maximum FROM observation o WHERE series_id in (" + tslistStr + ") AND ts between ? AND ?) " +
					"UNION (SELECT MAX(val_cd) as maximum FROM missing_values WHERE series_id in (" + tslistStr + ") AND ts between ? AND ?))";
			System.out.println("A maxVal tsQuery ha van windowstart s windowEND: " + tsQuery);		
			 tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, windowStart);
			tsStmt.setInt(2, windowEnd);
			tsStmt.setInt(3, windowStart);
			tsStmt.setInt(4, windowEnd);
			}
			ResultSet tsRs = tsStmt.executeQuery();

			if(tsRs.next()) {
				maxVal = tsRs.getFloat(1);
			} 

		} catch (SQLException e) {
			e.printStackTrace();
			return 0;
		}
		System.out.println("A maxVal: " + maxVal);		
		return maxVal;
	}
	
	
	public float getMinValueFromMoreTs(ArrayList<Integer> timeseriesList, int windowStart, int windowEnd) {
		float minVal = 0;
		PreparedStatement tsStmt;
		StringBuffer sbf = new StringBuffer();
		for(Integer i: timeseriesList) {
			sbf.append(Integer.toString(i) + ",");
		}
		
		String tslistStr = sbf.toString();
		tslistStr = tslistStr.substring(0, tslistStr.length() - 1);

		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return 0;
			}
			
			if(windowStart == -1 && windowEnd == -1) {
				String tsQuery = "SELECT MIN(minimum) FROM (SELECT MIN(val) as minimum FROM observation o WHERE series_id in (" + tslistStr + "))";
				System.out.println("A minVal tsQuery: " + tsQuery);				
				 tsStmt= conn.prepareStatement(tsQuery);
			} else {
			
			String tsQuery = "SELECT MIN(minimum) FROM ((SELECT MIN(val) as minimum FROM observation o WHERE series_id in (" + tslistStr + ") AND ts between ? AND ?)" +
					"UNION (SELECT MIN(val_cd) as minimum FROM missing_values WHERE series_id in (" + tslistStr + ") AND ts between ? AND ?))";
			System.out.println("A minVal tsQuery ha van windowstart s windowEND: " + tsQuery);		
			tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, windowStart);
			tsStmt.setInt(2, windowEnd);
			tsStmt.setInt(3, windowStart);
			tsStmt.setInt(4, windowEnd);
			}
			ResultSet tsRs = tsStmt.executeQuery();

			if(tsRs.next()) {
				minVal = tsRs.getFloat(1);
			} 

		} catch (SQLException e) {
			e.printStackTrace();
			return 0;
		}
		
		System.out.println("A minVal: " + minVal);		
		return minVal;
	}
	
	public float getMaxValue(int ts_id, int windowStart, int windowEnd) {
		float maxVal = 0;

		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return 0;
			}
			String tsQuery = "SELECT MAX (val) FROM observation where series_id = ? AND ts between ? AND ?";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, ts_id);
			tsStmt.setInt(2, windowStart);
			tsStmt.setInt(3, windowEnd);
			ResultSet tsRs = tsStmt.executeQuery();

			if(tsRs.next()) {
				maxVal = tsRs.getFloat(1);
			} 

		} catch (SQLException e) {
			e.printStackTrace();
			return 0;
		}

		return maxVal;
	}

	public float getMinVal(int ts_id, int windowStart, int windowEnd) {
		float minVal = 0;
		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return 0;
			}
			String tsQuery = "SELECT MIN (val) FROM observation where series_id = ? AND ts between ? AND ?";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, ts_id);
			tsStmt.setInt(2, windowStart);
			tsStmt.setInt(3, windowEnd);
			ResultSet tsRs = tsStmt.executeQuery();

			if(tsRs.next()) {
				minVal = tsRs.getFloat(1);
			} 

		} catch (SQLException e) {
			e.printStackTrace();
			return 0;
		}
		
	return minVal;
	}
	
	
	public int getLastTS(int ts_id, int windowStart, int windowEnd) {
		int lastTS = 0;

		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return 0;
			}
			String tsQuery = "SELECT MAX (ts) FROM observation where series_id = ? AND ts between ? AND ?";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, ts_id);
			tsStmt.setInt(2, windowStart);
			tsStmt.setInt(3, windowEnd);
			ResultSet tsRs = tsStmt.executeQuery();

			if(tsRs.next()) {
				lastTS = tsRs.getInt(1);
			} 

		} catch (SQLException e) {
			e.printStackTrace();
			return 0;
		}

		return lastTS;
	}
	
	public int getFirstTS(int ts_id, int windowStart, int windowEnd) {
		int firstTS = 0;

		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return 0;
			}
			String tsQuery = "SELECT MIN (ts) FROM observation where series_id = ? AND ts between ? AND ?";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, ts_id);
			tsStmt.setInt(2, windowStart);
			tsStmt.setInt(3, windowEnd);
			ResultSet tsRs = tsStmt.executeQuery();

			if(tsRs.next()) {
				firstTS = tsRs.getInt(1);
			} 

		} catch (SQLException e) {
			e.printStackTrace();
			return 0;
		}

		return firstTS;
	}
	

	public int getLastTSandValue(int ts_id, int windowStart, int windowEnd) {
		int lastTS = 0;

		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return 0;
			}
			String tsQuery = "SELECT MAX (ts) FROM observation where series_id = ? AND ts between ? AND ?";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, ts_id);
			tsStmt.setInt(2, windowStart);
			tsStmt.setInt(3, windowEnd);
			ResultSet tsRs = tsStmt.executeQuery();

			if(tsRs.next()) {
				lastTS = tsRs.getInt(1);
			} 

		} catch (SQLException e) {
			e.printStackTrace();
			return 0;
		}

		return lastTS;
	}

	public int getRowNum(int ts_id) {
		int rowNum = 0;

		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return 0;
			}
			String tsQuery = "SELECT COUNT (*) FROM observation WHERE series_id = ?";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, ts_id);
			ResultSet tsRs = tsStmt.executeQuery();

			if(tsRs.next()) {
				rowNum = tsRs.getInt(1);
			} 
		} catch (SQLException e) {
			e.printStackTrace();
			return 0;
		}
		return rowNum;
	}

	public Matrix extractTimeSeriesData( int rows, int ts_id) {
		Matrix m = new Matrix(rows, 1);
		int iterator = 0;
		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return null;
			}

			if(checkRownum(ts_id, rows)) {
				System.out.println("Querying database for ID "+ ts_id);
				String tsQuery = "SELECT val FROM observation WHERE series_id = ? AND rownum < ? ORDER BY ts";
				PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
				tsStmt.setInt(1, ts_id);
				tsStmt.setInt(2, rows+1);
				ResultSet tsRs = tsStmt.executeQuery();

				System.out.println("Loding data into a matrix...");

				while (iterator < rows) {
					tsRs.next();
					m.set(iterator, 0, tsRs.getFloat(1));
					//log("The value extracted from the database: " + tsRs.getfloat(1));
					iterator++;
				}
			}

		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}

		//log("The matrix m from extractTimeSeriesData: " + m + "\n\n");
		return m;
	}

	public boolean checkRownum(int ts_id, int rownum) {
		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return false;
			}

			String tsQuery = "SELECT COUNT (*) FROM observation WHERE series_id = ?";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, ts_id);
			ResultSet tsRs = tsStmt.executeQuery();

			if(tsRs.next()) {
				if(rownum > tsRs.getInt(1)) {
					throw new RuntimeException("You gave " +rownum+ " as rownum, but there is only " +tsRs.getInt(1) + " entries in this timeseries: "+ts_id+".");
				} else {
					return true;
				}
			} else {
				return false;
			}

		} catch (SQLException e) {
			e.printStackTrace();
			return false;
		}
	}

	public ArrayList<TimeSeriesSet> selectTimeSeriesSets() {

		ArrayList<TimeSeriesSet> list = new ArrayList<TimeSeriesSet>();

		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return null;
			} 

			String tsQuery = "SELECT id, label01, granul FROM sets";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			ResultSet tsRs = tsStmt.executeQuery();

			while (tsRs.next()) {
				TimeSeriesSet set = new TimeSeriesSet(tsRs.getInt(1), tsRs.getString(2), tsRs.getInt(3));
				list.add(set);
			}

		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}

		return list;

	}

	public ArrayList<TimeSeries> selectTimeSeriesFromSet(int src_set) {
		
		ArrayList<TimeSeries> list = new ArrayList<TimeSeries>();
		
		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return null;
			} 
			
			String tsQuery = "SELECT id, label01 FROM series s INNER JOIN sets_series ss ON s.id=ss.src_series WHERE src_set = ?";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, src_set);
			ResultSet tsRs = tsStmt.executeQuery();
			
			while (tsRs.next()) {
				TimeSeries ts = new TimeSeries(tsRs.getInt(1), tsRs.getString(2));
				list.add(ts);
			}
			
		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}

		return list;
		
	}
	
	public ArrayList<Integer> selectTimeSeriesFromSameSets(int ts_id) {
		int set_id = 0;
		ArrayList<Integer> list = new ArrayList<Integer>();

		try {
			Connection conn = db.newConnection();

			if (conn==null) {
				System.out.println("Connection null");
				return null;
			} 

			String tsQuery = "SELECT src_set FROM sets_series WHERE src_series = ?";
			PreparedStatement tsStmt= conn.prepareStatement(tsQuery);
			tsStmt.setInt(1, ts_id);
			ResultSet tsRs = tsStmt.executeQuery();

			if(tsRs.next()) {
				set_id = tsRs.getInt(1);
			} 

			String tsQuery1 = "SELECT id FROM series s INNER JOIN sets_series ss ON s.id=ss.src_series WHERE src_set = ?";
			PreparedStatement tsStmt1= conn.prepareStatement(tsQuery1);
			tsStmt1.setInt(1, set_id);
			ResultSet tsRs1 = tsStmt1.executeQuery();

			while (tsRs1.next()) {
				list.add(tsRs1.getInt(1));			
				//log("The value extracted from the database: " + tsRs.getfloat(1));
			}

		} catch (SQLException e) {
			e.printStackTrace();
			return null;
		}

		return list;
	}
		
	public Controller getC() {
		return c;
	}

	public void setC(Controller c) {
		this.c = c;
	}

	private void log(String text) {
		if (shouldLog) {
			System.out.println(text);
			 c.log(text, Color.BLUE);
		}
	}
	
	private void logdc(String text) {
		if (shouldLog) {
			System.out.println(text);
			dc.log(text, Color.BLUE);
		}
	}

	public DisplayController getDc() {
		return dc;
	}

	public void setDc(DisplayController dc) {
		this.dc = dc;
	}
	

	
}
