package db;

import java.util.ArrayList;
import model.Observation;
import cede.Matrix;

/**
 * Make transformation operation on data
 */

public class DataTransformer {

	public static boolean shouldLog = true;
	DataLoader dataloader = DataLoader.getInstance();
	
	
	
	/**
	 * Make the interpolation for the missing timestamps.
	 * 
	 * @param ts_start: first timestamp NOT to include
	 * @param ts_end: first timestamp NOT to include after ts_start
	 * @param val_start: value belongs to ts_start
	 * @param val_end: value belongs to ts_start
	 * @return ArrayList, what contains the missing timestamps and belonging interpolated values
	 */
	public ArrayList<Observation> linearInterpol(int ts_start, float val_start, int ts_end, float val_end, int gran, int windowStart) {

		int missingObsNum = (int) ((ts_end-ts_start)/gran-1);
		float increment = (val_end-val_start)/(missingObsNum+1);
		ArrayList<Observation> list = new ArrayList<Observation>();

		for(int i = 0; i < missingObsNum; i++) {
			Observation mo = new Observation();
			mo.setIndex((ts_start-windowStart)/gran+(i+1));
			mo.setTimeStamp(ts_start+(i+1)*gran);
			mo.setValue(val_start+(i+1)*increment);
			list.add(mo);
		}
		return list;
	}
	
	public Matrix createDataMatrix(int rows, int[] ts_id_array) {
		int m = ts_id_array.length;
		Matrix dataMatrix = new Matrix(1,1);

		for(int i = 0; i < m; i++) {

			Matrix dataMatrixTEMP = dataloader.extractTimeSeriesData(rows, ts_id_array[i]);
			if(i == 0) {
				dataMatrix = dataMatrixTEMP;
			} else {
				dataMatrix = dataMatrix.appendRight(dataMatrixTEMP);
			}
		}

		return dataMatrix;
	}

	public Matrix create100ColumnsMatrix(int rows) {

		Matrix dataMatrix = new Matrix(1,1);

		for(int i = 100; i < 200; i++) {

			Matrix dataMatrixTEMP = dataloader.extractTimeSeriesData(rows, i);
			if(i == 100) {
				dataMatrix = dataMatrixTEMP;
			} else {
				dataMatrix = dataMatrix.appendRight(dataMatrixTEMP);
			}
		}

		return dataMatrix;
	}

	public Matrix loadPreprocessedTimeSeriesWithCorrection(int ts_id, int windowStart, int windowEnd) {
		
		Matrix m = dataloader.loadPreprocessedTimeSeries(ts_id, windowStart, windowEnd);
		Matrix m1 = smoothTempTimeSeries(m, windowStart, windowEnd);
		
		return m1;
	}
	
	public Matrix createMatrixFromCorrelatedTS(int[] ts_id_array, int windowStart, int windowEnd) {
		log("Creating preprocessed data matrix starts... ");
		Matrix dataMatrix = new Matrix(1,1);
		Matrix dataMatrixTEMP;

		for(int i = 0; i < ts_id_array.length; i++) {
			
			dataMatrixTEMP = dataloader.loadPreprocessedTimeSeries(ts_id_array[i], windowStart, windowEnd);
			
			if(i == 0) {
				dataMatrix = dataMatrixTEMP;
			} else {
				dataMatrix = dataMatrix.appendRight(dataMatrixTEMP);
			}
		}

		return dataMatrix;
	}

	public Matrix createMatrixFromCorrelatedTSWithCorrections(int[] ts_id_array, int windowStart, int windowEnd) {
		log("Creating preprocessed data matrix with corrections starts... ");
		Matrix dataMatrix = new Matrix(1,1);
		Matrix dataMatrixTEMP;

		for(int i = 0; i < ts_id_array.length; i++) {
			int gran = dataloader.queryingForGranularity(ts_id_array[i]);
			
			if(gran == 2) {
				Matrix m = dataloader.loadPreprocessedTimeSeries(ts_id_array[i], windowStart, windowEnd);
				dataMatrixTEMP = smoothTempTimeSeries(m, windowStart, windowEnd);
			} else {
				dataMatrixTEMP = dataloader.loadPreprocessedTimeSeries(ts_id_array[i], windowStart, windowEnd);
			}

			if(i == 0) {
				dataMatrix = dataMatrixTEMP;
			} else {
				dataMatrix = dataMatrix.appendRight(dataMatrixTEMP);
			}
		}

		return dataMatrix;
	}

	public Matrix smoothTempTimeSeries(Matrix m, int windowStart, int windowEnd) {
		
		Matrix doubleM = null;
		boolean startIsOdd = false;
		boolean endIsOdd = false;
		
		if(windowStart % 2 != 0) {
			startIsOdd = true;
		}
		
		if(windowEnd % 2 != 0) {
			endIsOdd = true;
		}
		
		//if one of them is odd, than the matrix should contains one more row
		if(startIsOdd || endIsOdd) {
			doubleM = new Matrix(2*m.getRows(), 1);
		}
		else {
			doubleM = new Matrix(2*m.getRows()-1, 1);
		}
		
		int i = 0;
		int j = 0;
		
		if(startIsOdd) {
			j = 1;
			doubleM.set(0, 0, windowStart);
		} 
		
		while(i < m.getRows()-1) {
			doubleM.set(j, 0, m.get(i, 0));
			j++;
			float average = (m.get(i, 0)+m.get(i+1, 0))/2;
			doubleM.set(j, 0, average);
			j++;
			i++;
		}
		
		doubleM.set(j, 0, m.get(i, 0));
		
		if(endIsOdd) {
			doubleM.set(doubleM.getRows()-1, 0, windowEnd);
		}

		return doubleM;
	}
	

	
	private static void log(String text) {
		if (shouldLog) {
			System.out.println(text);
		}
	}
	
}
