package gui;

import javax.swing.*;
import javax.swing.event.*;

import org.jfree.ui.RefineryUtilities;

import preprocessing.FindCorrelated;

import recovery.Recover;

import db.DataLoader;
import db.Database;

import java.awt.*;
import java.util.ArrayList;
import java.util.HashSet;


/**
 * Main Controller, the program can started to run from here. 
 * This is the controller is responsible for the recovery.
 *
 */

public class Controller implements ListSelectionListener {

	Database db;
	private int tab;
	static DataLoader loader;
	DisplayController dispContr;
	DecompositionController decompContr;
	JTablePanel tablePanel;
	BottomPanel bottomPanel;
	TopPanel topPanel;
	ArrayList<Integer> timeSeriesList = new ArrayList<Integer>();
	Recover r;
	int[] correlatedTS;


	public Controller() {
		db = Database.getInstance();
		loader = DataLoader.getInstance();
		loader.setC(this);
		dispContr = new DisplayController();
		decompContr = new DecompositionController();
	}

	public void findCorrelated(int ts_id_recov)  {

		FindCorrelated fc = new FindCorrelated();
		correlatedTS = fc.findCorreletedTS(ts_id_recov, 3, this);
		timeSeriesList.clear();
		for(int i = 0; i<3; i++) {
			timeSeriesList.add(correlatedTS[i]);
		}
		log("The most correlated time series are: " + correlatedTS[0] + ", " + correlatedTS[1] + ", " + correlatedTS[2] + ".", Color.BLUE);
		enableThirdStep();

	}

	public void deleteTestTs() {
		timeSeriesList.clear();
		topPanel.tf_timeSeriesList.setText("");
		topPanel.buttonDelete.setVisible(false);
	}

	public void preProcessedClicked(int ts_id_recov) {
		int lastTS = Integer.parseInt(topPanel.lastTimeStamp.getText());
		if(!topPanel.rbPartly.isSelected()) {   
			if(topPanel.rbFind.isSelected()) {
				Recover r = new Recover(ts_id_recov, correlatedTS, this);
				r.createPreProcessedDataMatrix();
			} else {							
				int[] array = createTestTSarray();
				r = new Recover(ts_id_recov, array, this);
				r.createPreProcessedDataMatrix();
			}

		} else {
			if("".equals(topPanel.tf_windowStart.getText()) || "".equals(topPanel.tf_windowEnd.getText())) {
				log("Give window start and window end!", Color.RED);
			} else {		
				if(Integer.parseInt(topPanel.tf_windowStart.getText()) >= lastTS || Integer.parseInt(topPanel.tf_windowEnd.getText()) > lastTS) {
					log("Window start or window end is bigger than the last timestamp in the reference timeseries! Pleas give it again!", Color.RED);
				} else if(Integer.parseInt(topPanel.tf_windowStart.getText()) >= Integer.parseInt(topPanel.tf_windowEnd.getText())) {
					log("Window start is bigger or equal than window end! Pleas give it again!",  Color.RED);
				} else {
					Integer windowStart = Integer.parseInt(topPanel.tf_windowStart.getText());
					Integer windowEnd = Integer.parseInt(topPanel.tf_windowEnd.getText());
					if(topPanel.rbFind.isSelected()) {
						r = new Recover(ts_id_recov, windowStart, windowEnd, correlatedTS, this);
						r.createPreProcessedDataMatrix();
					} else {							
						int[] array = createTestTSarray();
						r = new Recover(ts_id_recov, windowStart, windowEnd, array, this);
						r.createPreProcessedDataMatrix();
					}
				}
			}
		}
		topPanel.buttonPreProcess.setEnabled(false);
		topPanel.buttonRecover.setEnabled(true);
	}

	public void recover() {
		r.recover();
		ArrayList<Integer> list = new ArrayList<Integer>();
		list.add(r.getTs_id_recov());
		SliderFrame demo = new SliderFrame("Recovered TS", list, "recov" , r.getWindowStart(), r.getWindowEnd());
//		System.out.println("Windowstart: " + r.getWindowStart() + " Windowend: " + r.getWindowEnd());
        demo.pack();
   
        RefineryUtilities.centerFrameOnScreen(demo);
        demo.setVisible(true);
        
        ArrayList<Integer> listRefAndTest = new ArrayList<Integer>();
        
        listRefAndTest.add(r.getTs_id_recov());
        listRefAndTest.addAll(timeSeriesList);
        
        
        SliderFrame demo1 = new SliderFrame("Original TS", listRefAndTest, "ref_test" , r.getWindowStart(), r.getWindowEnd());
        demo1.pack();
        RefineryUtilities.positionFrameOnScreen(demo1, 20, 50);
        demo1.setVisible(true);
        
        topPanel.buttonRecover.setEnabled(false);
	}
	
	public void displayMissingValsInRefandTest() {
		
	}
	
	public void displayrecovered() {
		
	}

	public int[] createTestTSarray() {
		int[] array = new int[timeSeriesList.size()];
		int j = 0;
		for(Integer i: timeSeriesList) {
			array[j] = i;
			j++;
		}
		return array;
	}

	public void testTimeSereiesSelected(String ts_id) {
		if(topPanel.rbTestTs.isSelected() && !topPanel.rbFind.isSelected()) {

			topPanel.buttonDelete.setVisible(true);
			timeSeriesList.add(Integer.parseInt(ts_id));
			removeDuplicates();
			if(!checkNumber()) {
				timeSeriesList.remove((Object)Integer.parseInt(ts_id));
				log("Only 5 different timeseries can be chosen!",  Color.RED);
			} else {

				StringBuffer buff = new StringBuffer();
				for(Integer i: timeSeriesList) {
					buff.append(Integer.toString(i) + ",");
				}
				String text = buff.toString();
				topPanel.tf_timeSeriesList.setText(text);
			}

			enableThirdStep();
		}
	}

	public void refTimeSeriesSelected(String ts_id) {


		if(!topPanel.rbTestTs.isSelected() && !topPanel.rbFind.isSelected()) {

			topPanel.tsid.setText(ts_id);
			topPanel.tsid.setVisible(true);
			topPanel.buttonReset.setVisible(true);
			topPanel.rbTestTs.setEnabled(true);
			topPanel.rbFind.setEnabled(true);
			int lastTimeStamp = loader.getLastTS(Integer.parseInt(ts_id));
			topPanel.label.setText("is selected and it's last timestamp is: ");
			topPanel.lastTimeStamp.setText(Integer.toString(lastTimeStamp));
			topPanel.label.setVisible(true);
			topPanel.lastTimeStamp.setVisible(true);		

		} 
	}


	public void removeDuplicates() {
		//Create a hashSet from the ArrayList because it's removes the duplicate items
		HashSet<Integer> hashSet = new HashSet<Integer>(timeSeriesList);

		//Then create a new list from the hashset
		ArrayList<Integer> list = new ArrayList<Integer>(hashSet) ;


		if(timeSeriesList.size() != list.size()) {
			timeSeriesList.clear();
			timeSeriesList = list;
		}

	}

	public boolean checkNumber() {
		if(timeSeriesList.size() > 5) {			
			return false;
		} else
			return true;
	}

	private void enableThirdStep() {
		topPanel.rbPartly.setEnabled(true);
		topPanel.windowStartLabel.setEnabled(true);
		topPanel.windowEndLabel.setEnabled(true);
		topPanel.rbWhole.setEnabled(true);
	}

	@Override
	public void valueChanged(ListSelectionEvent e) {

		int selectedRow = e.getFirstIndex();//tablePanel.getTable().getSelectedRow();
		//System.out.println("selectedRow" + selectedRow);
		String s = (String) tablePanel.getTable().getValueAt(selectedRow, 0);
		int ts_id = Integer.parseInt(s);

		System.out.println("Selected: " + ts_id);
	}
	
	public void display(int ts_id_recov) {
		
		
	}


//	public int[] getTokenizedText(String s) {
//		StringTokenizer st = new StringTokenizer(s, ",", false);
//		int size = st.countTokens();
//		int[] array = new int[size];
//
//		for(int i = 0; i <size; i++){
//			array[i] = Integer.parseInt(st.nextToken());
//			System.out.println(array[i]);
//		}
//		return array;
//	}

	public void setBottomPanel(BottomPanel bottomPanel) {
		this.bottomPanel = bottomPanel;
	}

	public void setTopPanel(TopPanel topPanel) {
		this.topPanel = topPanel;
	}

	public void setTablePanel(JTablePanel tablePanel) {
		this.tablePanel = tablePanel;
	}

	public void listClear() {
		timeSeriesList.clear();	
	}

	public void log(String text, Color color) {
		bottomPanel.appendLogText(text, color);
	}

	public int getTab() {
		return tab;
	}

	public void setTab(int tab) {
		this.tab = tab;
	}

	public DisplayController getDisplayController() {
		return dispContr;
	}

	public DecompositionController getDecompContr() {
		return decompContr;
	}

	

	public void cleanlog() {
		bottomPanel.log.setText("");
		
	}

	
}
