package gui;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.DefaultListSelectionModel;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

import db.DataLoader;

import model.TimeSeries;


public class JTablePanel extends JPanel implements TableModelListener {

	DisplayController dc;
	DataLoader loader = DataLoader.getInstance();
	JLabel label;
	private JTable table;
	ArrayList<TimeSeries> rowList;

	public JTablePanel(int setID, final DisplayController dc) {
		super(new BorderLayout());
		this.dc = dc;
		
		label = new JLabel("Time series: ");

		initTable(setID);

		TableColumn column1 = table.getColumnModel().getColumn(0);
		column1.setPreferredWidth(100);
		TableColumn column2 = table.getColumnModel().getColumn(1);
		column2.setPreferredWidth(400);

		table.setCellSelectionEnabled(false);
		table.setRowSelectionAllowed(true);
		table.setColumnSelectionAllowed(false);
		
		//table.getModel().addTableModelListener(this);
		
		dc.setTsTable(this);
		
		ListSelectionModel rowSelectionModel = table.getSelectionModel();
		rowSelectionModel.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
//		rowSelectionModel.addListSelectionListener(controller);
		rowSelectionModel.addListSelectionListener(new ListSelectionListener() {
			
			int lastSelected = -1;
			
			public void valueChanged(ListSelectionEvent e) {
				//Thread.dumpStack();
				int selectedRow = table.getSelectedRow();
				
				if (lastSelected==selectedRow || selectedRow==-1) {
					return;
				}
				
				lastSelected=selectedRow;
				
				String s = (String) table.getValueAt(selectedRow, 0);
//				controller.timeSeriesSelected(s);
				dc.updateTimeSeries(s);
				
				System.out.println("Selected: " + s);
				
				
			}

		});
		
		
		JScrollPane paneTemp = new JScrollPane(table);

		add(label, BorderLayout.NORTH);
		add(paneTemp, BorderLayout.CENTER);


	}
	
	public void setTable(JTable table) {
		this.table = table;
	}


	public void initTable(int setID) {
		
		rowList = loader.selectTimeSeriesFromSet(setID);
		
		Vector<String> columnNames = new Vector<String>();
		columnNames.addElement("ID");
		columnNames.addElement("Description");
		
		Vector<Vector> rows = new Vector<Vector>();
		for(TimeSeries ts: rowList) {
			Vector<String> rowElement = new Vector<String>();
			rowElement.add(0, Integer.toString(ts.getTs_id()));
			rowElement.add(1, ts.getDescription());
			rows.addElement(rowElement);
		}
		
		if (table==null)  {
			table = new JTable();
		}
		
		DefaultTableModel tableModel = new DefaultTableModel(rows, columnNames) {

		    @Override
		    public boolean isCellEditable(int row, int column) {
		       //all cells false
		       return false;
		    }
		};

		
		table.setModel(tableModel);			
//		table.tableChanged(e)
		table.repaint();
		
	}
	
	 public void tableChanged(TableModelEvent e) {
	        int row = e.getFirstRow();
	        int column = e.getColumn();
	        TableModel model = (TableModel)e.getSource();
	        String columnName = model.getColumnName(column);
	        Object data = model.getValueAt(row, column);
	      
	        // Do something with the data...
	    }

	
	public void refreshTable(int setID) {
	    DefaultTableModel contactTableModel = (DefaultTableModel) table.getModel();
	    contactTableModel.fireTableDataChanged();
	}

	public JTable getTable() {
		return table;
	}	
	
	class MyTableModel extends AbstractTableModel {

		 public String[] colNames = { "ID", "Description"};
		 
		 public Class[] colTypes = { Integer.class, String.class};
		 
		 public MyTableModel(Vector dataVector) {
		      super();
		      
		    }
		 
		
		
		@Override
		public int getColumnCount() {
			// TODO Auto-generated method stub
			return colNames.length;
		}

		@Override
		public int getRowCount() {
			// TODO Auto-generated method stub
			return 0;
		}

		@Override
		public Object getValueAt(int arg0, int arg1) {
			// TODO Auto-generated method stub
			return null;
		}
		
	}
}
