package gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.border.Border;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.jfree.chart.ChartColor;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.ui.RectangleInsets;

import db.DataLoader;

public class SliderPanel extends JPanel implements ChangeListener {

	private DataLoader loader;
	private JSlider slider;
	private XYPlot plot;
//	int maxTimestamp;
	float maxVal;
	float minVal;
	private String instruction;
	int windowStart;
	int windowEnd;
	JFreeChart chart;

	public SliderPanel(ArrayList<Integer> tsID_list, String instruction, int windowStart, int windowEnd) {
		super(new BorderLayout());
		loader = DataLoader.getInstance();
		this.instruction = instruction;
		this.windowStart = windowStart;
		this.windowEnd = windowEnd;
		maxVal = loader.getMaxValueFromMoreTs(tsID_list, windowStart, windowEnd);
		minVal = loader.getMinValueFromMoreTs(tsID_list, windowStart, windowEnd);
		final XYDataset dataset = createDataset(tsID_list);
		if("recov".equals(instruction)) {
			 chart = createChart(dataset, windowStart, windowEnd);
		} 
		else if("ref_test".equals(instruction)) {
			chart = createChart(dataset, windowStart, windowEnd);
		}
		else {
		 chart = createChart(dataset, 0, 2000);
		}
		final ChartPanel chartPanel = new ChartPanel(chart);
		chartPanel.setPreferredSize(new java.awt.Dimension(900, 400));
		chartPanel.setDomainZoomable(true);
		chartPanel.setRangeZoomable(true);
		Border border = BorderFactory.createCompoundBorder(
				BorderFactory.createEmptyBorder(4, 4, 4, 4),
				BorderFactory.createEtchedBorder()
				);
		chartPanel.setBorder(border);
		add(chartPanel);

		JPanel dashboard = new JPanel(new BorderLayout());
		dashboard.setBorder(BorderFactory.createEmptyBorder(0, 4, 4, 4));   

		
		
		if("recov".equals(instruction)) {
			slider = new JSlider(JSlider.HORIZONTAL, windowStart);
			slider.setMaximum(windowEnd);
			slider.setMinimum(windowStart);
		} else {
		slider = new JSlider(JSlider.HORIZONTAL, 0);
		this.windowEnd = getMaxTimestamp(tsID_list);
		slider.setMaximum(this.windowEnd); // last timestamp
		}
		slider.setPreferredSize(new java.awt.Dimension(800, 15));		
		slider.addChangeListener(this);
		dashboard.add(slider);
		add(dashboard, BorderLayout.SOUTH);
	}

	public int getMaxTimestamp(ArrayList<Integer> tsID_list) {
		int maxVal = 0;
		for(Integer i: tsID_list) {
			int val = loader.getLastTS(i);
			if(val > maxVal) {
				maxVal = val;
			}
		}
		return maxVal;
	}

	/**
	 * Creates a sample dataset.
	 * 
	 * @return a sample dataset.
	 */
	private XYSeries createSeries(int ts_id) {
		
		XYSeries timeseries = null;
		
		if("display".equals(instruction)) {
			timeseries  = loader.loadTimeSeriesForDisplay(ts_id, -1, -1);
		} 
		else if("ref_test".equals(instruction)) {
			timeseries = loader.loadTimeSeriesForDisplay(ts_id, windowStart, windowEnd);
		}
		else {
			timeseries  = loader.loadRecoveredTimeSeriesforDisplay(ts_id, windowStart, windowEnd);
		}
	
		return timeseries;
	}
	
	private XYDataset createDataset(ArrayList<Integer> tsID_list) {
		
		XYSeriesCollection dataset = new XYSeriesCollection();
		
		for(Integer i: tsID_list) {
			XYSeries series = createSeries(i);
			dataset.addSeries(series);
		}
	
		
//		
//		final XYSeries series1 = new XYSeries("First");
//		series1.add(1.0, 1.0);
//		series1.add(2.0, 4.0);
//		series1.add(3.0, 3.0);
//		series1.add(4.0, 5.0);
//		series1.add(5.0, 5.0);
//		series1.add(6.0, 7.0);
//		series1.add(7.0, 7.0);
//		series1.add(8.0, 8.0);
//
//		final XYSeries series2 = new XYSeries("Second");
//		series2.add(1.0, 5.0);
//		series2.add(2.0, 7.0);
//		series2.add(3.0, 6.0);
//		series2.add(4.0, 8.0);
//		series2.add(5.0, 4.0);
//		series2.add(6.0, 4.0);
//		series2.add(7.0, 2.0);
//		series2.add(8.0, 1.0);
//
//		final XYSeries series3 = new XYSeries("Third");
//		series3.add(3.0, 4.0);
//		series3.add(4.0, 3.0);
//		series3.add(5.0, 2.0);
//		series3.add(6.0, 3.0);
//		series3.add(7.0, 6.0);
//		series3.add(8.0, 3.0);
//		series3.add(9.0, 4.0);
//		series3.add(10.0, 3.0);
//
//		
//		dataset.addSeries(timeseries1);
////		dataset.addSeries(series1);
////		dataset.addSeries(series2);
////		dataset.addSeries(series3);

		return dataset;

	}

	/**
	 * Creates a chart.
	 * 
	 * @param dataset  the data for the chart.
	 * 
	 * @return a chart.
	 */
	private JFreeChart createChart(final XYDataset dataset, double lowerRange, double upperRange) {

		// create the chart...
		final JFreeChart chart = ChartFactory.createXYLineChart(
				"",      // chart title
				"Timestamps",                      // x axis label
				"Value",                      // y axis label
				dataset,                  // data
				PlotOrientation.VERTICAL,
				true,                     // include legend
				true,                     // tooltips
				false                     // urls
				);

		// NOW DO SOME OPTIONAL CUSTOMISATION OF THE CHART...
		chart.setBackgroundPaint(Color.lightGray);
		chart.setPadding(new RectangleInsets(10, 10, 10, 10));
		
		//	        final StandardLegend legend = (StandardLegend) chart.getLegend();
		//      legend.setDisplaySeriesShapes(true);

		// get a reference to the plot for further customisation...
		plot = chart.getXYPlot();
		plot.setRangePannable(false);
		plot.setDomainPannable(false);

		plot.setBackgroundPaint(Color.white);
		//    plot.setAxisOffset(new Spacer(Spacer.ABSOLUTE, 5.0, 5.0, 5.0, 5.0));
		plot.setDomainGridlinePaint(Color.lightGray);
		plot.setRangeGridlinePaint(Color.lightGray);
		plot.getDomainAxis().setRange(lowerRange, upperRange);
		
        final NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();
//      rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
      rangeAxis.setStandardTickUnits(NumberAxis.createStandardTickUnits());
//      rangeAxis.setAutoRangeIncludesZero(true);
//      rangeAxis.setAutoRange(true);
      rangeAxis.setRange((minVal-0.5f), (maxVal+0.5f));
      System.out.println("Minval: " + minVal + "and maxVal: " + maxVal);
    
		
//		TickUnits units = new TickUnits();
//		NumberTickUnit unit = new NumberTickUnit(100);
//		units.add(unit);
//		plot.getDomainAxis().setStandardTickUnits(units);
		
		
		XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer(true,false);
		
//		XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer(false,
//				true);

		renderer.setSeriesPaint(0, ChartColor.BLUE);
		renderer.setSeriesPaint(1, ChartColor.RED);
		renderer.setSeriesPaint(2, ChartColor.DARK_GREEN);
		renderer.setSeriesPaint(3, ChartColor.DARK_MAGENTA);
		renderer.setSeriesPaint(4, ChartColor.DARK_CYAN);
		renderer.setSeriesPaint(5, ChartColor.DARK_GRAY);	
		
		
//		final XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
//		renderer.setSeriesLinesVisible(0, true);
//		renderer.setSeriesShapesVisible(1, false);
		plot.setRenderer(renderer);

//		// change the auto tick unit selection to integer units only...
//		final NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();
//		rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
		
		 // customise the range axis...

		// OPTIONAL CUSTOMISATION COMPLETED.

		return chart;

	}

	@Override
	public void stateChanged(ChangeEvent e) {
		int value = this.slider.getValue();
		if(value+1000 < this.windowEnd) {
			plot.getDomainAxis().setRange(value, value+1000);
		} 
	}

}
