package gui;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.Vector;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import db.DataLoader;

import model.TimeSeries;
import model.TimeSeriesSet;

public class TableOfSets extends JPanel {

	DisplayController dc;
	DataLoader loader = DataLoader.getInstance();
	JLabel label;
	JTable setTable;
	ArrayList<TimeSeriesSet> rowList;

	public TableOfSets(final DisplayController dc) {
		super(new BorderLayout());
		this.dc = dc;
		label = new JLabel("Sets: ");

		rowList = loader.selectTimeSeriesSets();
		
		Vector<String> columnNames = new Vector<String>();
		columnNames.addElement("ID");
		columnNames.addElement("Description");
		columnNames.addElement("Granul");

		Vector<Vector> rows = new Vector<Vector>();
		for(TimeSeriesSet set: rowList) {
			Vector<String> rowElement = new Vector<String>();
			rowElement.add(0, Integer.toString(set.getTs_id()));
			rowElement.add(1, set.getLabel());
			rowElement.add(2, Integer.toString(set.getGranul()));
			rows.addElement(rowElement);
		}

		setTable = new JTable();
		
		DefaultTableModel tableModel = new DefaultTableModel(rows, columnNames) {

		    @Override
		    public boolean isCellEditable(int row, int column) {
		       //all cells false
		       return false;
		    }
		};

		setTable.setModel(tableModel);


		TableColumn column1 = setTable.getColumnModel().getColumn(0);
		column1.setPreferredWidth(100);
		TableColumn column2 = setTable.getColumnModel().getColumn(1);
		column2.setPreferredWidth(350);
		TableColumn column3 = setTable.getColumnModel().getColumn(2);
		column3.setPreferredWidth(100);

		setTable.setCellSelectionEnabled(false);
		setTable.setRowSelectionAllowed(true);
		setTable.setColumnSelectionAllowed(false);

		dc.setSetTable(this);

		ListSelectionModel rowSelectionModel = setTable.getSelectionModel();
		rowSelectionModel.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		rowSelectionModel.addListSelectionListener(new ListSelectionListener() {

			int lastSelected = -1;

			public void valueChanged(ListSelectionEvent e) {
				//Thread.dumpStack();
				int selectedRow = setTable.getSelectedRow();

				if (lastSelected==selectedRow) {
					return;
				}

				lastSelected=selectedRow;
				int id = Integer.parseInt((String) setTable.getValueAt(selectedRow, 0));

				System.out.println("Selected: " + id);
				
				dc.setSelected(id);

			}

		});
		
		

		JScrollPane paneTemp = new JScrollPane(setTable);

		add(label, BorderLayout.NORTH);
		add(paneTemp, BorderLayout.CENTER);

	}

}

