package gui;


import java.awt.Container;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import db.DataLoader;
import db.Database;


/**
 * This is the main view, contains the tabbedpane with the 3 tab.
 *
 */

public class UI extends JFrame {

	Controller c;
	DisplayController dispContr;
	DecompositionController decompContr;
	private Database db;
	DataLoader loader;


	public UI(Controller c, DisplayController dispContr, DecompositionController decompContr) {
		db = Database.getInstance();
		this.c = c;
		this.dispContr = dispContr;
		this.decompContr = decompContr;
		this.loader = DataLoader.getInstance();
		db.connectionTest();	
		initUI();
	}

	public final void initUI() {

		Container content = getContentPane();
		JTabbedPane tabbedPane = new JTabbedPane();

		RecoveryTabPanel recovPanel = new RecoveryTabPanel(c);
		DisplayTabPanel displayPanel = new DisplayTabPanel(c, dispContr);
		DecompositionTabPanel decompositionPanel = new DecompositionTabPanel(decompContr);

		tabbedPane.addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {
				if (e.getSource() instanceof JTabbedPane) {
					JTabbedPane pane = (JTabbedPane) e.getSource();
					System.out.println("Selected paneNo : " + pane.getSelectedIndex());
					c.setTab(pane.getSelectedIndex());
				}
			}
		});

		tabbedPane.addTab("Display", displayPanel);
		tabbedPane.addTab("Recover", recovPanel);
		tabbedPane.addTab("Decomposition", decompositionPanel);
		content.add(tabbedPane);

		pack();
		setVisible(true);

		setTitle("CeDRIT");
		setSize(1101, 740);
		setLocationRelativeTo(null);
		setDefaultCloseOperation(EXIT_ON_CLOSE);
	}





}
