package preprocessing;

import gui.Controller;

import java.awt.Color;
import java.util.ArrayList;

import model.Observation;

import cede.Matrix;
import db.DataInserter;
import db.DataLoader;
import db.DataTransformer;


/**
 * Class for preprocessing a time series. It makes extrapolition in both direction: forward and backward.
 * It makes interpolation for the missing timestamps.
 */


public class PreProcessor {

	int ts_id;
	int windowStart;
	int windowEnd;
	int lastTS;
	int firstTS;
	private DataLoader loader = DataLoader.getInstance();
	private DataInserter inserter = DataInserter.getInstance();
	private DataTransformer transformer;
	private Matrix m;
	private int gran;
	public static boolean shouldLog = true;
	Controller controller;


	public PreProcessor(int ts_id, int wStart, int wEnd, Controller controller) {

		this.ts_id = ts_id;
		this.gran = loader.queryingForGranularity(ts_id);
		//		System.out.println("The granularity is : " + gran);

		this.windowStart = wStart;
		this.windowEnd = wEnd;

//		it's necessary if somebody give an odd timestamp as windowStart or windowEnd, and the time series has granularity 2
		if(gran == 2) {
			if(wStart % 2 == 1) {
				this.windowStart++;
			}
			if(wEnd % 2 == 1) {
				this.windowEnd--;
			}
		}


		this.controller = controller;
		this.transformer = new DataTransformer();

		lastTS = loader.getLastTS(ts_id, windowStart, windowEnd);
		firstTS = loader.getFirstTS(ts_id, windowStart, windowEnd);

		//		System.out.println("A ts_id: " + ts_id);
		//		System.out.println("A lastTS: " + lastTS);


		if(lastTS < this.windowStart) {
			log("There isn't any timestamps in the recovery window you given, that's why the preprocessing is not possible. Please broaden it!", Color.RED); 
			throw new RuntimeException("lastTS is smaller then windowStart");

		} else {
			this.m = loader.loadTimeSeriesForPreprocessing(ts_id, windowStart, windowEnd);
			log("TimeSeries with id: " + this.ts_id + "is already loaded for preprocessing.", Color.BLUE);
		}			
	}


	public ArrayList<Observation> preprocessing() {

		log("Preprocessing of " + ts_id + " time series starts...", Color.BLUE);

		ArrayList<Observation> missingValues = new ArrayList<Observation>();

		
//		extrapolation forward, necessary: if the given windowStart is one of the missing values as well
		if(windowStart < firstTS) {
			float minVal = loader.getMinVal(ts_id, windowStart, windowEnd);
			ArrayList<Observation> extrapolBlockforepart = new ArrayList<Observation>();
			Observation mo = new Observation();
			mo.setIndex(0);
			mo.setTimeStamp(this.windowStart);
			mo.setValue(minVal);
			extrapolBlockforepart.add(mo);
			ArrayList<Observation> temp = transformer.linearInterpol(windowStart, minVal, firstTS, m.get(0, 1), gran, windowStart);
			extrapolBlockforepart.addAll(temp);
			missingValues = extrapolBlockforepart;

		}


//		interpolation of missing values
		int i = 0;

		while(i  < m.getRows()-1) {
			if(m.get(i+1, 0)-m.get(i, 0) > gran) {
				ArrayList<Observation> sub = transformer.linearInterpol((int) m.get(i, 0), m.get(i, 1), (int) m.get(i+1, 0), m.get(i+1, 1), gran, windowStart);
				missingValues.addAll(sub);

				i++;
			}
			else {
				i++;
			}			
		}


//		extrapolation bacward, necessary: if the last timestamp in that time series is smaller than the required windowend
		if(lastTS < windowEnd) {

			//			System.out.println("A lastTS: " + lastTS);
			//			System.out.println("A windowEnd: " + windowEnd);

			float maxVal = loader.getMaxValue(ts_id, windowStart, lastTS);
			ArrayList<Observation> extrapolBlock = transformer.linearInterpol((int) m.get(m.getRows()-1, 0), m.get(m.getRows()-1, 1), windowEnd, maxVal, gran, windowStart);
			Observation mo = new Observation();
			//			System.out.println("Az extrapolblock sizeja: " + extrapolBlock.size());
			if(extrapolBlock.size() == 0) {
				mo.setIndex((windowEnd-windowStart)/gran);
			} else {
				mo.setIndex(extrapolBlock.get(extrapolBlock.size()-1).getIndex()+1);
			}
			mo.setTimeStamp(windowEnd);
			mo.setValue(maxVal);
			extrapolBlock.add(mo);
			missingValues.addAll(extrapolBlock);
		}


//		insert tha inter and extrapolated values into the database
		inserter.insertORupdateMissingValues(ts_id, missingValues);

//		gives back the list of missingvalues, it's important only by the preprocessing of the reference time series
		return missingValues;		
	}




	public void cleanMissingValues() {
		inserter.cleanMissingValueTable();
	}

	//	This method is for debugging only
	public void printList(ArrayList<Observation> list) {
		for(Observation mo: list) {
			System.out.println("-Index: " + mo.getIndex() + " -TimeStamp: " + mo.getTimeStamp() + " -Value: " + mo.getValue());
		}
	}

	private void log(String text, Color color) {
		if (shouldLog) {
			//			System.out.println(text);
			controller.log(text, color);


		}
	}

}
