package recovery;

import gui.Controller;

import java.awt.Color;
import java.util.ArrayList;

import model.MeasurePearson;
import model.Observation;

import preprocessing.FindCorrelated;
import preprocessing.PreProcessor;

import cede.CentroidDecomposition;
import cede.Matrix;
import cede.TimeWatch;
import db.DataInserter;
import db.DataLoader;
import db.DataTransformer;
import db.Database;


/**
 * Class for missing value recovery. The different constructors are because of the different set-ups, can be used for the recovery.
 */


public class Recover {

	int ts_id_recov;
	int windowStart;
	int windowEnd;
	int lastTS;
	int gran;
	int[] testTimeSeries;
	int[] granTestTimeSeries;

	ArrayList<Observation> missingIndecesANDTimeStamps;
	Matrix preProcessedData;
	private Database db;
	private DataLoader loader;
	private DataInserter inserter;
	private DataTransformer transformer;
	public static boolean shouldLog = true;
	public static StringBuffer logtext = new StringBuffer();
	Controller controller;


	public Recover(int ts_id_recov,  int windowStart, int windowEnd, int[] otherTimeSeries, Controller controller) {
		this.ts_id_recov = ts_id_recov;	
		this.controller = controller;
		this.windowStart = windowStart;
		this.windowEnd = windowEnd;
		testTimeSeries = otherTimeSeries;

		initDB();

//		System.out.println("A 4 paramteres konstruktornl ts_id_recov: " + ts_id_recov + " windowStart: " + this.windowStart + " windowEnd: " + this.windowEnd + "gran: " + gran);
		printCorrelatedTS();
		fillGranularities();
	}

	public Recover(int ts_id_recov, int windowStart,int windowEnd, Controller controller) {
		this.ts_id_recov = ts_id_recov;	
		this.controller = controller;
		initDB();
		this.windowStart = windowStart;
		this.windowEnd = windowEnd;
//		System.out.println("A 3 paramteres konstruktornl ts_id_recov: " + ts_id_recov + " windowStart: " + this.windowStart + " windowEnd: " + this.windowEnd + "gran: " + gran);
		findCorrelatedTS(3);
		printCorrelatedTS();
		fillGranularities();
	} 

	public Recover(int ts_id_recov, int[] otherTimeSeries, Controller controller) {
		this.ts_id_recov = ts_id_recov;	
		this.controller = controller;
		initDB();
		windowStart = 0;
		windowEnd = loader.getLastTS(ts_id_recov);
		testTimeSeries = otherTimeSeries;

//		System.out.println("A 2paramteres konstruktornal ts_id_recov: " + ts_id_recov + " windowStart: " + windowStart + " windowEnd: " + windowEnd+ "gran: " + gran);
		printCorrelatedTS();
		fillGranularities();
	}

	public Recover(int ts_id_recov, Controller controller) {
		this.ts_id_recov = ts_id_recov;	
		this.controller = controller;
		initDB();
		windowStart = 0;
		windowEnd = loader.getLastTS(ts_id_recov);
//		System.out.println("A szimpla konstruktornal ts_id_recov: " + ts_id_recov + " windowStart: " + windowStart + " windowEnd: " + windowEnd+ "gran: " + gran);

		findCorrelatedTS(3);
		printCorrelatedTS();
		fillGranularities();
	}

	public void initDB() {
		db = Database.getInstance();
		db.connectionTest();
		loader = DataLoader.getInstance();
		inserter = DataInserter.getInstance();
		transformer = new DataTransformer();
		gran = loader.queryingForGranularity(ts_id_recov);

	}

	public void fillGranularities() {

		granTestTimeSeries = new int[testTimeSeries.length];
		for(int i = 0; i<testTimeSeries.length; i++) {
			//			TODO it throw an "open cursors exceeded error" exception, what I could not solve, that's why the hardcoded version is used
			//			int gr = loader.queryingForGranularity(testTimeSeries[i]);
			int gr = -1;
			if(testTimeSeries[i] < 179) {
				gr = 2;
			}
			else { 
				gr = 1;
			}
			granTestTimeSeries[i] = gr;
//			System.out.println("Granularity of " + testTimeSeries[i] + " is " + granTestTimeSeries[i]);
		}
	}

	public void findCorrelatedTS(int number) {
		FindCorrelated fc = new FindCorrelated();
		testTimeSeries = fc.findCorreletedTS(ts_id_recov, number, controller);
	} 

	public void preprocessing() {
		PreProcessor pp;

		log("Preprocessing of reference timeseries " + ts_id_recov + " starts...", Color.BLUE);

		inserter.cleanMissingValueTable();
		pp = new PreProcessor(ts_id_recov, windowStart, windowEnd, controller);
		missingIndecesANDTimeStamps = pp.preprocessing();

		log("Preprocessing of reference timeseries " + ts_id_recov + " is done.", Color.BLUE);

		for(int i = 0; i < testTimeSeries.length; i ++) {

			log("Preprocessing of test timeseries " + testTimeSeries[i] + " starts...", Color.BLUE);

			pp = new PreProcessor(testTimeSeries[i], windowStart, windowEnd, controller);
			pp.preprocessing();

			log("Preprocessing of test timeseries " + testTimeSeries[i] + " done.", Color.BLUE);

		}	
	}

	public void createPreProcessedDataMatrix()  {

		log("Creating preprocessed datamatrix starts ... ", Color.BLUE);

		Matrix recov_matrix = null;
		Matrix temp = null;

		preprocessing();

		String s = granCheck();

		if("normal".equals(s)) {
			recov_matrix = loader.loadPreprocessedTimeSeries(ts_id_recov, windowStart, windowEnd);
			//			log("A recov_matrix mrete: " + recov_matrix.getRows());

			temp = transformer.createMatrixFromCorrelatedTS(testTimeSeries, windowStart, windowEnd);
			//			log("A correlatedTimeSeriesek mrete: " + temp.getRows());
		} else if("testTScorrection".equals(s)) {
			recov_matrix = loader.loadPreprocessedTimeSeries(ts_id_recov, windowStart, windowEnd);
			temp = transformer.createMatrixFromCorrelatedTSWithCorrections(testTimeSeries, windowStart, windowEnd);

		} else if("refTScorrection".equals(s)) {
			recov_matrix = transformer.loadPreprocessedTimeSeriesWithCorrection(ts_id_recov, windowStart, windowEnd);
			temp = transformer.createMatrixFromCorrelatedTS(testTimeSeries, windowStart, windowEnd);
			//The indices went wrong because of the granularity correction
			for(Observation mo: missingIndecesANDTimeStamps) {
				mo.setIndex(2*mo.getIndex());
			}

		} else if("testANDrefTScorrection".equals(s)) { 
			recov_matrix = transformer.loadPreprocessedTimeSeriesWithCorrection(ts_id_recov, windowStart, windowEnd);
			temp = transformer.createMatrixFromCorrelatedTSWithCorrections(testTimeSeries, windowStart, windowEnd);
			for(Observation mo: missingIndecesANDTimeStamps) {
				mo.setIndex(2*mo.getIndex());
			}
		}

		preProcessedData = recov_matrix.appendRight(temp);		
		log("Preprocessed datamatrix is done. ", Color.BLUE);
		//		System.out.println(preProcessedData);
	}


	public void recover() {

		log("Recovery starts... ", Color.BLUE);

		TimeWatch watch = TimeWatch.start();

		doCDandRewriteMissingVals();

		log("Recovery is done in : " + watch.time() + "ms", Color.BLUE);
		
		for(Observation mo: missingIndecesANDTimeStamps) {
			int index = mo.getIndex();
			float recovVal = preProcessedData.get(index, 0);
			mo.setValue(recovVal);
		}

		inserter.insertRecoveredValue(ts_id_recov, missingIndecesANDTimeStamps);
		log("Recovered values are inserted in the DB, displaying starts... ", Color.BLUE);
	}

	public void doCDandRewriteMissingVals() {

		int i = 1;
		double difference = Double.MAX_VALUE;


		while (difference > Math.pow(10, -4) && i < 50) {

//			System.out.println("Difference: " + difference);
			System.out.println("The iteration number : " + i );

			CentroidDecomposition cd = new CentroidDecomposition();

			Matrix clone = preProcessedData.clone();	
			Matrix prevState = preProcessedData.getCol(0).clone();
			cd.decompose(clone, preProcessedData.getCols()-1);
			Matrix approx = cd.getApprox();

			difference = Math.abs(preProcessedData.getCol(0).frobNorm()-approx.getCol(0).frobNorm());

			for(Observation mo: missingIndecesANDTimeStamps) {
				int index = mo.getIndex();
				float val = approx.get(index, 0);
				preProcessedData.set(index, 0, val);
			}		
			if(i>0){
				difference = Math.abs(preProcessedData.getCol(0).frobNorm()-prevState.frobNorm());
			}

			i++;
		}
	}


	//	granularity correction, to make possible the recovery on reference and test time series with different granularities
	public String granCheck() {
		String granCheck = "";
		boolean sameGran1 = false;
		boolean sameGran2 = false;
		boolean differentGran = false;

		double sum = 0;
		for(int i = 0; i<granTestTimeSeries.length; i++) {
			sum += granTestTimeSeries[i];
		}

		if(sum/testTimeSeries.length == 1) {
			sameGran1=true;
		} else if(sum/testTimeSeries.length == 2) {
			sameGran2=true;
		} else {
			differentGran = true;
		}

		if(gran == 1) {
			if(sameGran1) {
				granCheck = "normal";
			} else if(sameGran2) {
				granCheck = "testTScorrection";
			} else if(differentGran) {
				granCheck = "testTScorrection";
			}

		} else if(gran == 2) {
			if(sameGran1) {
				granCheck = "refTScorrection";
			} else if(sameGran2) {
				granCheck = "normal";
			} else if(differentGran) {
				granCheck = "testANDrefTScorrection";
			}
		}

		return granCheck;
	}


	private void log(String text, Color color) {
		if (shouldLog) {
			//			System.out.println(text);
			controller.log(text, color);
		}
	}

	// for debugging
	public void printListMO(ArrayList<Observation> list) {
		for(Observation mo: list) {
			System.out.println("-Index: " + mo.getIndex() + " -TimeStamp: " + mo.getTimeStamp() + " -Value: " + mo.getValue());
		}
	}

	//	for debugging
	public void printListMeasure(ArrayList<MeasurePearson> list) {
		for(MeasurePearson m: list) {
			System.out.println("-TS_ID: " + m.getTs_id() + " -Correlation: " + m.getPearson());
		}
	}

	public void printCorrelatedTS() {
		log("The most correlated time serieses are: ", Color.BLUE);
		for(int i = 0; i < testTimeSeries.length; i++) {
			log(Integer.toString(testTimeSeries[i]), Color.BLUE);
		}
	}

	public int[] getCorrelatedTimeSeries() {
		return testTimeSeries;
	}

	public int getTs_id_recov() {
		return ts_id_recov;
	}

	public void setTs_id_recov(int ts_id_recov) {
		this.ts_id_recov = ts_id_recov;
	}

	public int getWindowStart() {
		return windowStart;
	}

	public void setWindowStart(int windowStart) {
		this.windowStart = windowStart;
	}

	public int getWindowEnd() {
		return windowEnd;
	}

	public void setWindowEnd(int windowEnd) {
		this.windowEnd = windowEnd;
	}




}
